@@ -35,6 +35,7 @@ class FileOrSequence( object ):
    
    def __init__( self, filename_or_sequence ):      
       self.fos = filename_or_sequence
+      self.line_no = None
       
    def __iter__( self ):
       if isinstance( self.fos, str ):
@@ -44,10 +45,13 @@ class FileOrSequence( object ):
             lines = open( self.fos )
       else:
          lines = self.fos
+      self.line_no = 1
       for line in lines:
          yield line
+         self.line_no += 1
       if isinstance( self.fos, str ):
          lines.close()
+      self.line_no = None
          
    def __repr__( self ):
       if isinstance( self.fos, str ):
@@ -56,6 +60,12 @@ class FileOrSequence( object ):
       else:
          return "<%s object, connected to %s >" % (
             self.__class__.__name__, repr( self.fos ) )   
+            
+   def get_line_number_string( self ):
+      if isinstance( self.fos, str ):
+         return "line %d of file %s" % ( self.line_no, self.fos )
+      else:
+         return "line %d" % self.line_no
 
 class GeneratedSequence( object ):
    """GeneratedSequence is a little helper class to get "respawnable"
@@ -71,9 +81,6 @@ class GeneratedSequence( object ):
    def __iter__( self ):
       return self.generator( *self.args )
 
-_re_attr_main = re.compile( "\s*(\w+)[\s=]+(.*)" )
-
-
 #########################
 ## Features
 #########################
@@ -137,6 +144,8 @@ class GenomicFeature( object ):
          self.iv.strand, frame, attr_str ) ) + "\n"
          
 
+_re_attr_main = re.compile( "\s*(\w+)[\s=]+(.*)" )
+_re_attr_empty = re.compile( "^\s*$" )
 
 def parse_GFF_attribute_string( attrStr, extra_return_first_value=False ):
    """Parses a GFF attribute string and returns it as a dictionary.
@@ -148,7 +157,7 @@ def parse_GFF_attribute_string( attrStr, extra_return_first_value=False ):
       attrStr = attrStr[:-1]
    d = {}
    for (i, attr) in itertools.izip( itertools.count(), attrStr.split( ";" ) ):
-      if attr == "":
+      if _re_attr_empty.match( attr ):
          continue
       if attr.count( '"' ) not in ( 0, 2 ):
          raise ValueError, "The attribute string seems to contain mismatched quotes."
@@ -197,7 +206,7 @@ class GFF_Reader( FileOrSequence ):
             iv = GenomicInterval( seqname, int(start)-1, int(end), strand )
          f = GenomicFeature( name, feature, iv )
          if score != ".":
-            score = int( score )
+            score = float( score )
          if frame != ".":
             frame = int( frame )
          f.source = source
@@ -290,7 +299,6 @@ class FastaReader( FileOrSequence ):
          if line.startswith( ">" ):
             if seqname is not None:
                seqlengths[ seqname ] = length
-               print seqname, length
             mo = _re_fasta_header_line.match( line )
             seqname = mo.group(1)
             length = 0
@@ -476,9 +484,13 @@ class SAM_Reader( FileOrSequence ):
    def __iter__( self ):
       for line in FileOrSequence.__iter__( self ):
          if line.startswith( "@" ):
-	    # do something with the header line
-	    continue
-         algnt = SAM_Alignment( line )
+            # do something with the header line
+            continue
+	 try:
+            algnt = SAM_Alignment( line )
+	 except ValueError, e:
+	    e.args = e.args + ( self.get_line_number_string(), )
+	    raise
          yield algnt
        
 
